import mitt from 'mitt';
import { reactive } from 'vue';
import WebGL2WaveRender from './wave-view/render-wave';

export const emitter = mitt();


/**
 * @description 用于记载全局的一些对象，以便在不同组件中比较 ID
 * 
 */
export const globalLookup = reactive({
    // 所有的顶层文件
    topModules: [],
    // 当前选中的 信号，也就是 tree-view 左列的，默认是第一个。不可复选。
    currentModule: undefined,

    /**
     * @description 当前被选中的波形，用于进行高效的增删改查
     * @type {Set<WaveRenderItem>}
     */
    currentWires: new Set(),

    /**
     * @description 当前被选中的波形，用于 sidebar 的渲染视图
     * 详细请见设计文档：https://nc-ai-lab.feishu.cn/wiki/Fy3ZwtbYbiatmxkhOp2cyHSFnlw
     * @type {WaveRenderSidebarItem[]}
     */
    currentWiresRenderView: [],

    /**
     * @description 用于存储当前时间点的波形的值，Key 为 波形的 link，值为 当前左侧渲染的值，没有就是 x
     * @type {Record<string, BigInt | string>}
     */
    currentSignalValues: {},
    
    // 当前 ns 数（或者 ps）
    currentTime: 0,

    // 模拟器的版本，比如如果使用 iverilog 那么就是 Icarus Verilog
    version: '',
    // 创建时间
    date: '',
    // 状态 默认是 simulation
    status: '',
    // 单位时间
    timescale: '',
    // 初始时间，一般都是 0
    t0: 0,
    // 最长持续时间
    maxTime: 0n,
    time: 0,

    // 优化视图的最大公约数
    tgcd: null,
    
    /**
     * @description 就是 signalValues
     * @type {Record<string, ChangoItem>}
     */
    chango: {},
    
    // 初始化时会被定义
    render: () => {},
    waveRender: undefined,
    pstate: undefined,

    xScale: 1,

    initcurrentModule(module) {
        if (this.currentModule === undefined && module) {
            this.currentModule = module;
        }
        // 创造 parent
        if (module.body && module.body instanceof Array) {
            for (const childModule of module.body) {
                childModule.parent = module;
            }
        }
    },

    /**
     * @returns {WebGL2WaveRender}
     */
    getWaveRender() {
        return this.waveRender;
    }
});

export const globalStyle = reactive({
    timeScaleHeight: 30,
    sideBarPadding: 10,
    sideBarItemMargin: 5,
    vcdRenderPadding: 24,
    yOffset: 0,
});

export const globalSetting = reactive({
    language: 'zh',
    caseSensitivity: false,
    displayParentOnly: false,
    displaySignalHeight: 30,
    searchMode: 'so',        // so, mo, sm
    searchScope: ['wire', 'reg', 'integer'],
    displaySignalInfoScope: ['width'],
    HorizontalScalingRatio: 1,
    HorizontalRollRatio: 1,
    VerticalRollRario: 1,
    minGridWidth: 300,
    prerender: false,
    renderAnimation: false,
});

function loadSetting() {
    const settingString = localStorage.getItem('setting')
    try {
        const setting = JSON.parse(settingString);
        return setting;
    } catch (error) {
        return undefined;
    }
}

export const storedSetting = loadSetting();
if (storedSetting) {
    for (const key of Reflect.ownKeys(storedSetting)) {
        const value = storedSetting[key];
        if (value !== undefined) {
            globalSetting[key] = value;
        }
    }
}


/**
 * 
 * @returns {WaveRenderSidebarItem[]}
 */
function getCurrentWiresRenderView() {
    return globalLookup.currentWiresRenderView;
}

